#Author: Shuai Wang, Josee Dupuis
#Version: v1
#Date: 04/05/2016
#this function conducts association test of haplotype on cohort level, using zaykin's method adapted to account for familial correlation for cohorts with family data.
#formula: specify the formula as phenotype~covariates
#mygeno: allele-based genotype file, with the 1st column being the subject identifier
#restrict_hap: the haplotype name list you want to run the analysis
#id: the subject identifier
#mylabels: the haplotype name list of mygeno
#pheno: the phenotype file containing the subject identifer, the phenotype and covariates at least
#fullkins: the kinship matrix. When fullkins=NULL, it treats the subjects as unrelated. This function analyzes unrelated data by default, unless you provide with a kinship matrix
#cutoff: the haplotype frequency threshold below which the haplotypes would be collapsed, to stablize the computation. If no collapsing is preferred, it could be certainly set to 0.


zaykinfam=function(formula, mygeno,  restrict_hap, id="id", mylabels, pheno, fullkins=NULL, cutoff=0.005){
library(haplo.stats)

if (!is.data.frame(mygeno)){
    stop("mygeno must be a data frame!")
}

names(mygeno)[-1]=as.vector(apply(as.matrix(mylabels), 1, function(x){c(paste(x, "_A1", sep=""), paste(x, "_A2", sep=""))}))
restrict_hap_name=as.vector(apply(as.matrix(restrict_hap), 1, function(x){c(paste(x, "_A1", sep=""), paste(x, "_A2", sep=""))}))
mygeno_filter=mygeno[, c(id, names(mygeno)[names(mygeno) %in% restrict_hap_name])]

geno_pheno=merge(mygeno_filter, pheno, by=id)

pheno2=geno_pheno[, names(pheno)]
mygeno2=geno_pheno[, names(mygeno_filter)]

recode=mygeno2[,-1]

happrob <- haplo.em(recode,locus.label=restrict_hap)
geno <- setupGeno(recode, miss.val=c(NA),restrict_hap)
tmphap <- matrix(0,length(happrob$subj.id),length(happrob$hap.prob))

#haplotype to remove:
if (sum(happrob$hap.prob<=cutoff)>1){
hap_rm=which(happrob$hap.prob<=cutoff)
#keep the haplotype at least:
 hap_keep=which(happrob$hap.prob>cutoff)
#and
 hap_last=which(happrob$hap.prob==max(happrob$hap.prob[happrob$hap.prob<=cutoff]))
if (length(hap_last)>1){
     hap_last=sample(hap_last,1)
}
hap_rm=hap_rm[hap_rm!=hap_last]

happrob$hap.prob[hap_last]=1-sum(happrob$hap.prob[hap_keep])
happrob$hap.prob=happrob$hap.prob[c(hap_keep, hap_last)]
happrob$haplotype=happrob$haplotype[c(hap_keep, hap_last), ]
happrob$hap1code=ifelse(happrob$hap1code %in% hap_rm, hap_last, happrob$hap1code)
happrob$hap2code=ifelse(happrob$hap2code %in% hap_rm, hap_last, happrob$hap2code)







tmphap[cbind(1:nrow(tmphap),happrob$hap1code)] <- tmphap[cbind(1:nrow(tmphap),happrob$hap1code)]+ happrob$post
tmphap[cbind(1:nrow(tmphap),happrob$hap2code)] <-tmphap[cbind(1:nrow(tmphap),happrob$hap2code)]+ happrob$post
tmphap=tmphap[, -hap_rm]
} else{
    
    


    tmphap[cbind(1:nrow(tmphap),happrob$hap1code)] <- tmphap[cbind(1:nrow(tmphap),happrob$hap1code)]+ happrob$post
    tmphap[cbind(1:nrow(tmphap),happrob$hap2code)] <-tmphap[cbind(1:nrow(tmphap),happrob$hap2code)]+ happrob$post
}


#Next is to collapse the id with multiple haplotype rows
finalhap <- matrix(NA,length(table(happrob$subj.id)),length(happrob$hap.prob))
for(l in 1:ncol(tmphap)) {
  finalhap[,l] <- tapply(tmphap[,l],happrob$subj.id,sum)
}

myhap <- finalhap[,order(happrob$hap.prob, decreasing=T)]
colnames(myhap) <- paste("hap",1:ncol(finalhap),sep="")
myform <- ""
if ((grepl( "-1", deparse(formula))==TRUE)|(grepl( "- 1", deparse(formula))==TRUE)){
    for(kk in 1:ncol(finalhap)) myform <- paste(myform," + hap",kk,sep="")
} else {
    for(kk in 2:ncol(finalhap)) myform <- paste(myform," + hap",kk,sep="")
}
myform=paste("~.", myform, sep="")
fdata <- cbind(pheno2,myhap)
#names(fdata)[1] <- "id"

fform2 <- update(formula, eval(parse(text=myform)))

HAPPROB=cbind(happrob$haplotype, happrob$hap.prob);
names(HAPPROB)[ncol(HAPPROB)]="hap.prob";
HAPPROB2=HAPPROB[order(HAPPROB$hap.prob, decreasing=T),];
HAPPROB2$hapcat=1:nrow(HAPPROB2);


if (!is.null(fullkins)){
    library(coxme)
    fform2=update(fform2, eval(parse(text=paste("~.+(1|", id, ")", sep=""))))
    fit2lme <- lmekin(fform2,data=fdata,varlist=fullkins);
    res=list(estimate=fit2lme$coefficients$fixed, variance=fit2lme$var, haplofreq=HAPPROB2)
} else {
    fit2lm <- lm(fform2,data=fdata);
    res=list(estimate=summary(fit2lm)$coefficients, variance=summary(fit2lm)$cov.unscaled*(summary(fit2lm)$sigma)^2, haplofreq=HAPPROB2)
}

return(res)
}
